@@ -5,7 +5,7 @@
 
    Copyright (C) 1997-2005 Luke Howard
    Copyright (C) 2007 West Consulting
-   Copyright (C) 2007, 2008, 2009, 2010, 2011 Arthur de Jong
+   Copyright (C) 2007, 2008, 2009, 2010, 2011, 2012 Arthur de Jong
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -89,6 +89,7 @@ static void cfg_defaults(struct ldap_config *cfg)
   cfg->ldc_threads=5;
   cfg->ldc_uid=NOUID;
   cfg->ldc_gid=NOGID;
+  cfg->ldc_ignorecase=0;
   for (i=0;i<(NSS_LDAP_CONFIG_URI_MAX+1);i++)
   {
     cfg->ldc_uris[i].uri=NULL;
@@ -214,8 +215,19 @@ static void add_uris_from_dns(const char *filename,int lnr,
       *nxt='\0';
       nxt++;
     }
-    /* add the URI */
-    mysnprintf(buf,sizeof(buf),"ldap://%s",hostlist);
+    /* if port is 636, use ldaps:// URI */
+    if ((strlen(hostlist)>4)&&(strcmp(hostlist+strlen(hostlist)-4,":636")==0))
+    {
+      hostlist[strlen(hostlist)-4]='\0';
+      mysnprintf(buf,sizeof(buf),"ldaps://%s",hostlist);
+    }
+    else
+    {
+      /* strip default port number */
+      if ((strlen(hostlist)>4)&&(strcmp(hostlist+strlen(hostlist)-4,":389")==0))
+        hostlist[strlen(hostlist)-4]='\0';
+      mysnprintf(buf,sizeof(buf),"ldap://%s",hostlist);
+    }
     log_log(LOG_DEBUG,"add_uris_from_dns(): found uri: %s",buf);
     add_uri(filename,lnr,cfg,buf);
     /* get next entry from list */
@@ -852,6 +864,11 @@ static void cfg_read(const char *filename,struct ldap_config *cfg)
       get_gid(filename,lnr,keyword,&line,&cfg->ldc_gid);
       get_eol(filename,lnr,keyword,&line);
     }
+    else if (strcasecmp(keyword,"ignorecase")==0)
+    {
+      get_boolean(filename,lnr,keyword,&line,&cfg->ldc_ignorecase);
+      get_eol(filename,lnr,keyword,&line);
+    }
     /* general connection options */
     else if (strcasecmp(keyword,"uri")==0)
     {
